﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

/*
   Uno script che implementa la dinamica di una particella 
   implementata con Vertet integration e Position Based dynamics.
   Questo script va associato ad ogni particella presente nel gioco.
   NB: è solo un esercizio. Usando Unity, si può attivare la dinamica semplicemente associando un RigidBody ad un gameObject.
*/

public class DinamicaParticella : MonoBehaviour
{
    /* stato della particella (con Verlet) */
    public Vector3 pos;   // posizione attuale di questa particella
    Vector3 oldPos; // posizione di questa particella al fotogramma precedente

    /* var temporanee */
    Vector3 force;

    /* "constanti" della particella  */
    public float drag = 0.01f;  // lose 1% of speed per second
    public float mass = 2.0f;
    public bool isFixed = false;
    Vector3 fixedPos; 

    // unity chiama Start (su ogni gameObject) all'inizio del gioco
    void Start()
    {
        // eredita la "nostra"  posizione dal resto del game engine
        pos = transform./*global*/position;

        // init di Verlet (per oggetti che partono da fermi, cioè con speed=0)
        oldPos = pos;  

        fixedPos = pos;
    }

    public Vector3 Velocity()  {
        float dt = Time.fixedDeltaTime; // il dt usato al motore FISICO
        return (pos - oldPos) / dt;
    }

    // Unity chiama Update (su ogni game-object) subito prima di ogni RENDERING
    void Update(){
        // imponi la "nostra" posizione al resto del game engine (in spazio MONDO!)
        // in modo tale che la particella appaia a schermo nella posizione che abbiamo calcolato
        transform./*global*/position = pos;
    }

    void initForces(){
        force = new Vector3(0,0,0); // posizione di questa particella al fotogramma precedente
    }

    void addGravity(){
        force += Vector3.down * (9.8f*mass);
    }

    /* POSITIONAL CONSTRAINTS */
    void enforceAboveGround( ref Vector3 p ){
        float radius = 0.5f*transform.localScale.y;
        if (p.y<radius) p.y = radius;
    }

    void enforceFixed( ref Vector3 p ){
        p = fixedPos; // trivial ...
    }

    // Unity chiama FixedUpdate (su ogni game-object) per ogni passo della FISICA
    void FixedUpdate()
    {  
        fixedPos = transform.position; //HACK!

        initForces();

        addGravity();

        float dt = Time.fixedDeltaTime;

        float dump = 1-drag*dt; // approssimazione di (1-drag)^dt
        Vector3 acceleration = force / mass;
        Vector3 nextPos = (1+dump)*pos - dump*oldPos + acceleration * (dt*dt);

        enforceAboveGround( ref nextPos );
        if (isFixed) enforceFixed( ref nextPos );

        oldPos = pos;
        pos = nextPos;
        
    }
}
