﻿using System.Collections;
using System.Collections.Generic;
using UnityEngine;

public class MioOggettoRigido : MonoBehaviour
{
    
    Vector3 deltaPos;
    Quaternion deltaRot;

    public GameObject go; // l'oggetto rigido grafico associato a questo oggetto

    Vector3 averageVelocity(){
        DinamicaParticella[] particles = GetComponentsInChildren<DinamicaParticella>();
        Vector3 result = new Vector3(0,0,0);
        float totMass = 0;
        foreach (var p in particles) {
            result += p.mass * p.Velocity();
            totMass += p.mass;
        }
        return result / totMass;
    }

    Vector3 currentBarycenter()  {
        DinamicaParticella[] particles = GetComponentsInChildren<DinamicaParticella>();
        Vector3 result = new Vector3(0,0,0);
        float totMass = 0;
        foreach (var p in particles) {
            result += p.mass * p.transform.localPosition;
            totMass += p.mass;
        }
        return result / totMass;
    }

    /* trova una rotazione che segue le particelle
       (solo al fine di disegnare l'oggetto rigido nell'orientamento 
       determinato dalle particelle) */
    Quaternion currentRotation() {


        DinamicaParticella[] particles = GetComponentsInChildren<DinamicaParticella>();
        // ipotesi, ho almeno tre particelle, e non sono colienari
        // TODO: verifica delle ipotesi

        Vector3 bary = currentBarycenter();
        
        // posizione delle particelle (nel rendering)
        Vector3 pa = particles[0].transform.localPosition;
        Vector3 pb = particles[1].transform.localPosition;
        Vector3 pc = particles[2].transform.localPosition;

        // dal triangolo (pa, pb, pc) troviamo una matrice di rotazione,
        // cioe' un sistema di assi orto normale 

        // scelta arbitaria: l'asse delle X va da pa verso pb
        Vector3 asseX = (pb-pa).normalized;  
        // scelta arbitaria: l'asse delle Y è ortogonale al triangolo pa,pb,pc (sarà ortogonale anche a asseX)
        Vector3 asseY = Vector3.Cross(  pc-pa , asseX ).normalized;  
        // asse delle z, per definizione...
        Vector3 asseZ = Vector3.Cross( asseX, asseY );
        
        // passo da matrice avente colonne asseX, asseY, asseZ a quaternione
        Quaternion result = Quaternion.LookRotation(asseZ, asseY); 

        return result;
    }

    // Start is called before the first frame update
    void Start()
    {
        deltaPos = currentBarycenter() - go.transform.localPosition;   
        deltaRot = currentRotation() * Quaternion.Inverse( go.transform.localRotation )  ;
    }

    // Update is called once per frame
    void Update()
    {
        go.transform.localPosition = currentBarycenter() - deltaPos;
        go.transform.localRotation =  Quaternion.Inverse(deltaRot) * currentRotation() ;
    }
}
